import 'dart:async';

import 'package:ably_flutter_plugin/ably_flutter_plugin.dart';
import 'package:ably_flutter_plugin/src/impl/realtime/realtime.dart';

import '../../spec/spec.dart' show Connection, ConnectionState, ErrorInfo;
import '../platform_object.dart';

class ConnectionPlatformObject extends PlatformObject implements Connection {
  Realtime realtimePlatformObject;

  ConnectionPlatformObject(this.realtimePlatformObject) : super() {
    state = ConnectionState.initialized;
    on().listen((ConnectionStateChange event) {
      if (event.reason?.code == ErrorCodes.authCallbackFailure) {
        realtimePlatformObject.awaitAuthUpdateAndReconnect();
      }
      state = event.current;
    });
  }

  @override
  Future<int> createPlatformInstance() async =>
      await realtimePlatformObject.handle;

  @override
  ErrorInfo errorReason;

  @override
  String id;

  @override
  String key;

  @override
  String recoveryKey;

  @override
  int serial;

  @override
  ConnectionState state;

  @override
  Stream<ConnectionStateChange> on([ConnectionEvent connectionEvent]) {
    return listen(PlatformMethod.onRealtimeConnectionStateChanged)
        .map((connectionEvent) => connectionEvent as ConnectionStateChange)
        .where((connectionStateChange) =>
            connectionEvent == null ||
            connectionStateChange.event == connectionEvent);
  }

  @override
  Future<void> close() => realtimePlatformObject.close();

  @override
  Future<void> connect() => realtimePlatformObject.connect();

  @override
  Future<int> ping() {
    // TODO: implement ping
    return null;
  }
}
